import unittest

from pepperedform.renderutils import (sequence_getter, mapping_getter,
        make_start_token, FormRendererState, FormRenderer)


class TestRenderUtils(unittest.TestCase):

    def _get_mock_getter(self):
        def _mock_getter(items, name, if_missing=None):
            if items:
                return items.get(name, if_missing)
            return if_missing
        return _mock_getter

    def _get_renderer(self, **kwargs):
        return FormRenderer(**kwargs)

    def _get_renderer_state(self, defaults, errors=None, renderer=None):
        if not renderer:
            renderer = self._get_renderer()
        return renderer.form_renderer_state(defaults, errors)

    def test_push_id_stack(self):
        """ Stack should grow to 2 elements. """
        state = self._get_renderer_state({})
        state = state.mapping_descend('user')
        state = state.mapping_descend('identity')
        self.assertEqual(len(state.id_stack), 2)

    def test_build_id(self):
        """ Id stack should be glued together with spaces. """
        state = self._get_renderer_state({})
        state = state.mapping_descend('user')
        state = state.mapping_descend('identity')
        name = 'first_name'
        id_ = state.build_id(name)
        self.assertEqual(id_, 'user_identity_first_name')


    def test_sequence_getter_missing(self):
        items = None
        if_missing = tuple()
        result = sequence_getter(items, None, if_missing=if_missing)
        self.assertIs(result, if_missing,
                msg='Should return if_missing if items is falsy.')

    def test_sequence_getter(self):
        items = '15'
        if_missing = tuple()
        result = sequence_getter(items, None, if_missing=if_missing)
        self.assertEqual(result, '15',
                msg='Should return items if items is not falsy.')

    def test_mapping_getter_missing(self):
        items = {}
        if_missing = tuple()
        name = 'day'
        result = mapping_getter(items, name, if_missing=if_missing)
        self.assertIs(result, if_missing,
                msg='Should return if_missing if name not in items.')

    def test_mapping_getter(self):
        items = {
            'day': '15'
        }
        if_missing = tuple()
        name = 'day'
        result = mapping_getter(items, name, if_missing=if_missing)
        self.assertEqual(result, '15',
                'Should return value for key name in items if it exists.')

    def test_make_start_token(self):
        name = 'days'
        type_ = 'sequence'
        token = make_start_token(name, type_)
        self.assertEqual(token, 'days:sequence')

    def test_field_value(self):
        day_value = 1
        name = 'day'
        state = self._get_renderer_state({
            name: day_value
        })
        value = state.field_value(name)
        self.assertTrue(value, day_value)

    def test_sequence_descend(self):
        defaults = {
            'users': [{
                'name': u'Ian'
            },]
        }
        state = self._get_renderer_state(defaults)
        new_state = list(state.sequence_descend('users'))[0]
        self.assertEqual(new_state.field_value(None), defaults['users'][0])

    def test_field_error_missing(self):
        state = self._get_renderer_state({}, {})
        name = 'day'
        error = state.field_error(name)
        self.assertTrue(not error, 'No error should exist')

    def test_field_error(self):
        day_error = 'Today is the wrong day'
        name = 'day'
        state = self._get_renderer_state({
        }, {
            name: day_error
        })
        error = state.field_error(name)
        self.assertTrue(error and error == day_error)



"""
Non-mako utility functions used during rendering.
"""
from types import StringTypes
from webhelpers.misc import NotGiven
from webhelpers.html.builder import HTML


tag = HTML.tag
" Shorthand for webhelpers tag generator. "


default_field_part_orders = {
    'checkbox_field': ('input', 'label', 'help', 'example', 'error'),
    None: ('label', 'help', 'example', 'input', 'error')
}
"""
Map from field type to list of field part names.
The None key designates the default order since not all field types will have an entry.

Overwrite these in the render_kwargs.
"""


default_css_classes = {
    'field_errors': 'field_errors',
    'password_field': 'password_field',
    'file_field': 'file_field',
    'checkbox_field': 'checkbox_field',
    'textarea_field': 'textarea_field',
    'hidden_field': 'hidden_field',
    'text_field': 'text_field',
    'single_select_field': 'single_select_field',
    'single_select_field': 'single_select_field',
    'single_select_list_field': 'single_select_list_field',
    'single_select_list_item': 'single_select_list_item',
    'multiple_select_field': 'multiple_select_field',
    'multiple_select_list_field': 'multiple_select_list_field',
    'multiple_select_list_item': 'multiple_select_list_item',
    'repeater': 'repeater',
    'template': 'template',
    'adder': 'adder',
    'remover': 'remover',
    'repetition': 'repetition',
    'repetitions': 'repetitions',
    'field_label': 'field_label',
    'field_help': 'field_help',
    'field_example': 'field_example',
    'clear': 'clear',
    'primary_form_action': 'primary_form_action',
    'secondary_form_action': 'secondary_form_action',
    'field_has_error': 'field_has_error',
}
"""
Notable css classes indexed by key.

Overwrite these in the render kwargs.
"""


def sequence_getter(items, name, if_missing=None):
    """ A sequence should pass this function to its children to get their
    defaults or errors.

    @NOTE: The name here is ignored.
    """
    if items:
        return items
    return if_missing


def mapping_getter(items, name, if_missing=None):
    """  A mapping should pass this function to its children to get their
    defaults or errors.
    """
    if items:
        return items.get(name, if_missing)
    return if_missing


def make_start_token(name, type_):
    """ Makes the special token used by peppercorn to group things. """
    value = u'{0}:{1}'.format(name, type_)
    return value


def render_example(css_class, example):
    if type(example) not in (list, tuple):
        example = [example]
    return tag('span', *example, class_=css_class)


def render_help(css_class, help_):
    return tag('span', field_extras['help'], class_=css_class)


default_extra_renderers = {
    'example': render_example,
    'help': render_help,

}


class FormRenderer(object):
    """
    Use this to encapsulate configured across multiple renderings.
    """
    def __init__(self, css_classes=default_css_classes,
            extra_renderers=default_extra_renderers,
            field_part_orders=default_field_part_orders, closed=False):
        """
        ``css_classess``
          A dictionary containing css classes as either strings or sets which
          are mapped by special keys referenced by the functions in
          :module:``pepperedform.render``.  See ``default_css_classes`` for
          a list of the signifigant keys.

        ``extra_renderers``
          A dictionary containing functions for renderer extra parts of fields.
          These parts must be referenced in the ``field_part_orders`` in order
          to be renderer with the same key as their renderer.  Only ``example``
          and ``help`` are supported right now.

          ``field_part_orders``
          A dictionary mapping each field type to a list of field part keys
          that are used to determine in what order to render the parts of that
          type of field.  The ``None`` entry is used as the default field part
          order. For example if the label should occur before or after the
          input.  See ``default_field_part_orders`` for the supported list of
          field parts.

          ``closed``
          Close inputs for xhtml(4/5) or do not close them for regular
          html(4/5).  Nothing genius here.
        """

        self.css_classes = css_classes
        self.extra_renderers = extra_renderers
        self.closed = closed
        self.field_part_orders = field_part_orders

    def form_renderer_state(self, defaults, errors):
        return FormRendererState(self, defaults, errors, [], mapping_getter)


class FormRendererState(object):
    """
    This class encapsulates state for an intitial rendering but also for nested
    renderings.
    """
    def __init__(self, renderer, defaults, errors, id_stack, getter):
        self.renderer = renderer
        self.defaults = defaults
        self.errors = errors
        self.id_stack = id_stack
        self.getter = getter

    def is_closed(self):
        return self.renderer.closed


    def clone(self, renderer=NotGiven, defaults=NotGiven, errors=NotGiven,
            id_stack=NotGiven, getter=NotGiven):
        if renderer is NotGiven:
            renderer = self.renderer
        if defaults is NotGiven:
            defaults = self.defaults
        if errors is NotGiven:
            errors = self.errors
        if id_stack is NotGiven:
            id_stack = self.id_stack
        if getter is NotGiven:
            getter = self.getter
        return FormRendererState(renderer, defaults, errors, id_stack, getter)

    def rename_descend(self, name):
        defaults = self.field_value(name)
        errors = self.field_error(name)
        id_stack = self.push_id_stack(name)
        return FormRendererState(self.renderer, defaults, errors, id_stack,
                sequence_getter)

    def mapping_descend(self, name):
        defaults = self.field_value(name)
        errors = self.field_error(name)
        id_stack = self.push_id_stack(name)
        return FormRendererState(self.renderer, defaults, errors, id_stack,
                mapping_getter)

    def sequence_descend(self, name, getter=mapping_getter):
        defaults = self.field_value(name)
        errors = self.field_error(name)
        id_stack = self.push_id_stack(name)
        if defaults:
            for index, new_defaults in enumerate(defaults):
                # Errors must be either falsy or contain an entry
                # for every entry in the defaults list, if no error then the
                # entry should be None.
                if errors:
                    new_errors = errors[index]
                else:
                    new_errors = None
                new_id_stack = id_stack + [unicode(index)]
                yield FormRendererState(self.renderer, new_defaults,
                        new_errors, new_id_stack, sequence_getter)

    def determine_css_classes(self, class_name,
                add_css_classes=None):
        """
        Determine the css classes to use by first checking the dict at the
            `css_classes` key in render_kwargs and then checking defaults in this
            module.
        """
        css_classes = self.renderer.css_classes[class_name]
        # Wart to handle css classes as a string.
        if isinstance(css_classes, StringTypes):
            css_classes = set([css_classes])
        # Wart to handle css classes as a list or tuple.
        elif not isinstance(css_classes, set):
            css_classes = set(css_classes)
        if add_css_classes:
            css_classes.update(add_css_classes)
        return css_classes

    def determine_field_part_list(self, field_parts, field_part_order, field_type=None):
        """
        Determine the order of the parts, such as label and input, in the HTML.

        Note: This can be overridden in the render_kwargs by the field_type in a
            dict at the `field_part_orders` key, use the field_type `None` to match
            all unmatched field types. If no field part order is found in the
            dict in render kwargs then defaults in this module are used.

        A special field part exists to to inject a clearing div.
        """
        if not field_part_order:
            if field_type:
                field_part_order = self.renderer.field_part_orders.get(field_type, None)
            if not field_part_order:
                field_part_order = self.renderer.field_part_orders.get(None, None)
        if not field_part_order:
            # @TODO: Should we raise a warning here?
            field_part_order = []
        field_part_list = []
        for field_part_name in field_part_order:
            # Hack in order to clear floats.
            if field_part_name == 'clear':
                clear_class = self.determine_clear_class('clear')
                field_part_list.append(tag('div', '', class_=clear_class, _closed=True))
            elif field_part_name in field_parts:
                # Flatten iterable field parts into the existing list.
                if type(field_parts[field_part_name]) in (list, tuple):
                    field_part_list.extend(field_parts[field_part_name])
                else:
                    field_part_list.append(field_parts[field_part_name])
        return field_part_list

    def field_value(self, name, if_missing=None):
        return self.getter(self.defaults, name, if_missing=if_missing)

    def field_error(self, name, if_missing=None):
        return self.getter(self.errors, name, if_missing=if_missing)

    def push_id_stack(self, *names):
        return self.id_stack + [name for name in names if name]

    def build_id(self, *names):
        return u'_'.join(self.push_id_stack(*names))

    def render_field_errors(self, name):
        """
        Return an error messages div or None, each error is wrapped in a span.
        """
        errors = self.field_error(name)
        error_div = None
        if errors:
            error_spans = []
            if type(errors) in (list, tuple):
                for error in errors:
                    error_spans.append(tag('span', error))
            else:
                error_spans.append(tag('span', errors))
            css_class = _to_css_class(self.determine_css_classes('field_errors'))
            error_div = tag('div', *error_spans, class_=css_class)
        return error_div

    def determine_extra_renderer(self, extra_name):
        return self.renderer.extra_renderers.get(extra_name, None)

    def determine_extra_field_parts(self, field_parts, field_extras):
        """
        Build any extra field parts as necessary, such as an example or help.
        """
        extra_field_parts = {}
        if field_extras and 'example' in field_extras:
            render_example = self.determine_extra_renderer('example')
            if render_example:
                example_css_class = _to_css_class(
                    self.determine_css_classes('field_example'))
                extra_field_parts['example'] = render_example(
                        css_class=example_css_class,
                        example=field_extras['example'])
        if field_extras and 'help' in field_extras:
            render_help = self.determine_extra_renderer('help')
            if render_help:
                help_css_class = _to_css_class(self.determine_css_classes(
                        'field_help'))
                extra_field_parts['help'] = render_help(class_=help_css_class,
                        help_=field_extras['help'])
        return extra_field_parts


def _to_css_class(css_classes):
    return ' '.join(css_classes)



"""
Tools used when rendering the form.

The fields try to be flexible but if you want some heavy modification you are
probably better off just writing your own field and following or faking the
rules of form renderer state.


Field Extras
============

  * What is an example of an ``example`` extra?

    Date Ex: 10-11-1982

  * How would I use ``help`` extra?

    Use JS to roll the help text up and either show it when the input receives
    focus or when the help icon is clicked/hovered.

Field parts and their orders
============================

In order to support re-ordering of the structure of fields you can specify
the order of the parts.  The parts consist of input, label, help, example,
error and a special clear part.  The clear part just inject a div with
a clear class in order to allow floats to be cleared.  This part can occur
more than once in the given order.


Sequences
=========

Usually you want to use a list of dictionaries for this.

If you don't then only one value is available in the body.
"""
from mako.runtime import supports_caller, capture
from webhelpers.misc import NotGiven
from webhelpers.html.tags import select

from pepperedform.renderutils import (sequence_getter, make_start_token,
        _to_css_class, tag)


SEQUENCE = u'sequence'
MAPPING = u'mapping'
RENAME = u'rename'


def primary_form_action(context, state, value, css_classes=None,
        **attrs):
    """
    Primary submit button -- Use this for the main action of the form.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako tempalte.

    ``state``
      A ``FormRendererState`` instance.

    ``value``
      The value for the input tag.

    ``css_classes``
      A set of css classes to add to the css class resolved for this widget.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    attrs['class_'] = _to_css_class(state.determine_css_classes(
            'primary_form_action',
            add_css_classes=css_classes))
    attrs.setdefault('type', 'submit')
    closed = state.is_closed()
    context.write(tag('input', value=value, _closed=closed, **attrs))
    return ''


def secondary_form_action(context, state, value, tag_name='input',
        css_classes=None, **attrs):
    """
    Seconary submit button -- Use this for an action that is less significant
    than the primary action.


    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``value``
      Essentially the name of the action, will be set on the appropriate tag.

    ``tag_name``
      If "input" then an input tag is generated whose value is set to
      ``value``, otherwise a tag with this name is generated and its content
      is filled with the given value.

    ``css_classes``
      A set of css classes to add to the css class resolved for this widget.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    attrs['class_'] = _to_css_class(state.determine_css_classes(
            'secondary_form_action',
            add_css_classes=css_classes))
    if tag_name == 'input':
        attrs.setdefault('type', 'button')
        attrs.setdefault('value', value)
        attrs.setdefault('_closed', state.is_closed())
    else:
        # Content inside the tags.
        attrs.setdefault('c', value)
    context.write(tag(tag_name, **attrs))
    return ''


def hidden_field(context, state, name, value=NotGiven, id=NotGiven,
        field_part_order=None, field_classes=None, **attrs):
    """
    Standard input element of type hidden.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the hidden input tag.

    ``value``
      Optional override of the value attribute resolved by the ``state``.

    ``id``
      Optional override of the id generated from the ``name``.

    ``field_part_order``
      Override for the ``field_part_order`` determined by the ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    if value is NotGiven:
        value = state.field_value(name)
    if id is NotGiven:
        id = state.build_id(name)
    attrs['_closed'] = state.is_closed()
    attrs['class_'] = _to_css_class(state.determine_css_classes(
            'hidden_field'))
    field_parts = {}
    field_parts['input'] = tag('input', name=name, value=value, id=id, type='hidden', **attrs)
    error = state.render_field_errors(name)
    if error:
        field_parts['error'] = error
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    for field_part in field_part_list:
        context.write(field_part)
    return ''


def file_field(context, state, name, value=NotGiven, id=NotGiven,
        label=NotGiven, field_extras=None, field_part_order=None,
        field_classes=None, **attrs):
    """
    Standard input element of type file.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the file input tag.

    ``value``
      Optional override of the value attribute resolved by the ``state``.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Override for the ``field_part_order`` determined by the ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    field_parts = {}
    if value is NotGiven:
        value = state.field_value(name)
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    # If the label is None there will be no label.
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    attrs['_closed'] = state.is_closed()
    field_parts['input'] = tag('input', name=name, value=value, id=id, type='file', **attrs)
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes('field_has_error'))
        field_parts['error'] = error
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    css_class = _to_css_class(state.determine_css_classes(
            'file_field', add_css_classes=add_css_classes))
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def password_field(context, state, name, value=NotGiven, id=NotGiven,
        label=NotGiven, field_extras=None, field_part_order=None,
        field_classes=None, **attrs):
    """
    Standard input element of type password.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the input tag.

    ``value``
      Optional override of the value attribute resolved by the ``state``.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Override for the ``field_part_order`` determined by the ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    if value is NotGiven:
        value = state.field_value(name)
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    field_parts = {}
    if field_extras:
        _inject_field_extras(field_parts, field_extras)
    attrs['_closed'] = state.is_closed()
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    field_parts['input'] = tag('input', name=name, value=value, id=id, type='password', **attrs)
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes(
            'field_has_error'))
        field_parts['error'] = error
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    css_class = _to_css_class(state.determine_css_classes(
            'password_field', add_css_classes=add_css_classes))
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def text_field(context, state, name, value=NotGiven, id=NotGiven,
        label=NotGiven, field_extras=None, field_part_order=None,
        field_classes=None, **attrs):
    """
    Standard input element of type text.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the input tag.

    ``value``
      Optional override of the value attribute resolved by the ``state``.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Override for the ``field_part_order`` determined by the ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    if value is NotGiven:
        value = state.field_value(name)
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    attrs['_closed'] = state.is_closed()
    field_parts = {}
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    field_parts['input'] = tag('input', name=name, value=value, id=id, type='text', **attrs)
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes('field_has_error'))
        field_parts['error'] = error
    css_class = _to_css_class(state.determine_css_classes(
            'text_field', add_css_classes=add_css_classes))
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def checkbox_field(context, state, name, checked=NotGiven, id=NotGiven,
        label=NotGiven, field_extras=None, field_part_order=None,
        field_classes=None, **attrs):
    """
    Standard input element of type checkbox.

    DO NOT USE THIS FOR A LIST OF CHECKBOXES. THIS IS MEANT FOR A BOOLEAN FIELD.

    If you want to select zero or more of "things" with checkboxes then use
    ``multiple_select_list_field``.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the input tag.

    ``checked``
      True if the checkbox should be checked, otherwise it will not be checked.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Override for the ``field_part_order`` determined by the ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the input tag.

    """
    field_type = 'checkbox_field'
    if checked is NotGiven:
        checked = state.field_value(name, False)
    if checked:
        checked = 'checked'
    else:
        checked = None
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    attrs['_closed'] = state.is_closed()
    field_parts = {}
    field_parts['input'] = tag('input', name=name, value='1', checked=checked,
            id=id, type='checkbox', **attrs)
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes(
            'field_has_error'))
        field_parts['error'] = error
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list = state.determine_field_part_list(field_parts,
            field_part_order, field_type=field_type)
    css_class = _to_css_class(state.determine_css_classes(field_type,
            add_css_classes=add_css_classes))
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def textarea_field(context, state, name, value=NotGiven, id=NotGiven,
        label=NotGiven, field_extras=None, field_part_order=None,
        field_classes=None, **attrs):
    """
    Standard textarea element.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the textarea tag.

    ``value``
      Optional override of the value attribute resolved by the ``state``.

    ``id``
      Optional override of the id attribute generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Override for the ``field_part_order`` determined by the ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the textarea tag.

    """
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    field_type = 'textarea_field'
    if value is NotGiven:
        value = state.field_value(name, u'')
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    field_parts = {}
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    field_parts['input'] = tag('textarea', value, name=name, id=id, **attrs)
    error = state.render_field_errors(name)
    if error:
        add_css_classes.update(state.determine_css_classes(
            'field_has_error'))
        field_parts['error'] = error
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    css_class = _to_css_class(state.determine_css_classes(field_type,
            add_css_classes=add_css_classes))
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def single_select_field(context, state, name, options, value=NotGiven,
        id=NotGiven, label=NotGiven, field_extras=None, field_part_order=None,
        field_classes=None, **attrs):
    """
    Standard select element where multiple is not set.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the select tag.

    ``options``
      A list of length 2 tuples where the first entry is the value and the
      second entry is the label. Such as [('1', 'Yes'), ('2', 'No')].

    ``value``
      Optional override of the value resolved by the ``state``. This is matched
      against the first entry of the tuples in the supplied ``options`` to
      determine which option is selected.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Optional override for the ``field_part_order`` determined by the
      ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the select tag.

    """
    if value is NotGiven:
        value = state.field_value(name, NotGiven)
    if value is NotGiven:
        values = []
    else:
        values = [value]
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    attrs['_closed'] = state.is_closed()
    field_parts = {}
    field_parts['input'] = select(name, values, options, id=id)
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes(
            'field_has_error'))
        field_parts['error'] = error
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    css_class = _to_css_class(state.determine_css_classes(
            'single_select_field', add_css_classes=add_css_classes))
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def multiple_select_field(context, state, name, options,
        values=NotGiven, id=NotGiven, label=NotGiven, field_extras=None,
        field_part_order=None, field_classes=None, **attrs):
    """
    Standard select element where multiple is set to true.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for the select tag.

    ``options``
      An iterable of length 2 tuples where the first entry is the value and the
      second entry is the label. Such as [('1', 'Yes'), ('2', 'No')].

    ``values``
      Optional override of the values resolved by the ``state``. This is matched
      against the first entry of the tuples in the supplied ``options`` to
      determine which options are selected. This must be an iterable.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Optional override for the ``field_part_order`` determined by the
      ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on the select tag.

    """
    if values is NotGiven:
        values = state.field_value(name, [])
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    attrs['_closed'] = state.is_closed()
    field_parts = {}
    if label:
        field_parts['label'] = tag('label', label, for_=id)
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes('field_has_error'))
        field_parts['error'] = error
    field_parts['input'] = select(name, values, options, id=id, multiple=True)
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    css_class = _to_css_class(state.determine_css_classes(
            'multiple_select_field', add_css_classes=add_css_classes))
    context.write(tag('div', *field_part_list, class_=css_class))
    return ''


def multiple_select_list_field(context, state, name, options,
        values=NotGiven, label=NotGiven, id=NotGiven, field_extras=None,
        field_part_order=None, field_classes=None, **attrs):
    """
    Wrap a list of input elements of type checkbox.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for each checkbox.

    ``options``
      An iterable of length 2 tuples where the first entry is the checkbox
      value and the second entry is the checkbox label.
      Such as [('0', 'Bacon'), ('1', 'Eggs'), ('2', 'Toast')].

    ``values``
      Optional override of the values resolved by the ``state``. The values are
      used to determine which options are selected. This must be an
      iterable.

    ``label``
      Optional override of the label generated from the ``name``.

    ``id``
      Optional override of the id generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.

    ``field_part_order``
      Optional override for the ``field_part_order`` determined by the
      ``state``.

    ``**attrs``
      All remaining kwargs are set as attributes on each checkbox input tag.

    """
    if values is NotGiven:
        values = state.field_value(name, [])
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    closed = state.is_closed()
    attrs['_closed'] = closed
    start_token = make_start_token(name, SEQUENCE)
    field_parts = {}
    # Start list now because peppercorn stuff must always wrap around the
    # field parts.
    field_part_list = []
    field_part_list.append(tag('input', type='hidden', name='__start__', value=start_token, _closed=closed))
    if label:
        field_parts['label'] = tag('legend', label)
    item_css_class = _to_css_class(state.determine_css_classes(
            'multiple_select_list_item'))
    field_parts['input'] = []
    for index, option in enumerate(options):
        (option_value, option_label) = option
        if option_value in values:
            checked = 'checked'
        else:
            checked = None
        option_id = id + u'_' + unicode(option_value)
        field_parts['input'].append(tag('div',
                tag('input', name=name, value=option_value, id=option_id,
                        checked=checked, type='checkbox', **attrs),
                tag('label', option_label, for_=option_id), class_=item_css_class))
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes(
            'field_has_error'))
        field_parts['error'] = error
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list.extend(state.determine_field_part_list(field_parts, field_part_order))
    field_part_list.append(tag('input', type='hidden', name='__end__',
            value=start_token, _closed=closed))
    css_class = _to_css_class(state.determine_css_classes(
            'multiple_select_list_field', add_css_classes=add_css_classes))
    context.write(tag('div', tag('fieldset', *field_part_list), class_=css_class))
    return ''


def single_select_list_field(context, state, name, options,
        value=NotGiven, id=NotGiven, label=NotGiven, field_extras=None,
        field_part_order=None, field_classes=None, **attrs):
    """
    Wrap a list of input elements of type radio.

    ``context``
      A mako context, this won't be needed if this module is used as a
      namespace in a mako template.

    ``state``
      A ``FormRendererState`` instance.

    ``name``
      The name attribute for each radio.

    ``options``
      An iterable of length 2 tuples where the first entry is the value and the
      second entry is the label.
      Such as [('0', 'Red'), ('1', 'Green'), ('2', 'Blue')].

    ``value``
      Optional override of the value resolved by the ``state``. The value is
      used to determine which option is selected.

    ``id``
      Optional override of the id generated from the ``name``.

    ``label``
      Optional override of the label generated from the ``name``.

    ``field_extras``
      Optional dictionary mapping a field extra by name to content.  This must
      be set if you wish to use field extras.  See this module docstring for
      more information.  This applies to this field as a whole and not to
      individual inputs.

    ``field_part_order``
      Optional override for the ``field_part_order`` determined by the
      ``state``.

    ``field_classes``
      A iterable of css classes to add to the css class resolved for this field.

    ``**attrs``
      All remaining kwargs are set as attributes on each radio input tag.

    """
    if value is NotGiven:
        value = state.field_value(name)
    if id is NotGiven:
        id = state.build_id(name)
    if label is NotGiven:
        label = name.replace(u'_', u' ').title()
    attrs['_closed'] = state.is_closed()
    field_parts = {}
    if label:
        field_parts['label'] = tag('legend', label)
    item_css_class = _to_css_class(state.determine_css_classes(
            'single_select_list_item'))
    field_parts['input'] = []
    for index, option in enumerate(options):
        (option_value, option_label) = option
        checked = option_value == value
        if checked:
            checked = 'checked'
        else:
            checked = None
        option_id = id + u'_' + option_value
        field_parts['input'].append(tag('div',
                tag('input', name=name, value=option_value, id=option_id,
                        checked=checked, type='radio', **attrs),
                tag('label', option_label, for_=option_id),
                        class_=item_css_class))
    error = state.render_field_errors(name)
    if field_classes:
        add_css_classes = set(field_classes)
    else:
        add_css_classes = set()
    if error:
        add_css_classes.update(state.determine_css_classes(
                'field_has_error'))
        field_parts['error'] = error
    if field_extras:
        field_parts.update(state.determine_extra_field_parts(field_parts, field_extras))
    field_part_list = state.determine_field_part_list(field_parts, field_part_order)
    css_class = _to_css_class(state.determine_css_classes(
            'single_select_list_field', add_css_classes=add_css_classes))
    context.write(tag('div', tag('fieldset', *field_part_list), class_=css_class))
    return ''


@supports_caller
def rename(context, state, name=None):
    """
    Read the peppercorn documentation to understand why you need this.
    """
    closed = state.is_closed()
    value = make_start_token(name, RENAME)
    context.write(tag('input', type='hidden', name='__start__', value=value, _closed=closed))
    context['caller'].body(state.rename_descend(name))
    context.write(tag('input', type='hidden', name='__end__', value=value, _closed=closed))
    return ''


@supports_caller
def mapping(context, state, name=None):
    """
    Wrap a block of fields that should be serialized as a dictionary using
        peppercorn.

    The caller's body should take state as an argument.
    """
    closed = state.is_closed()
    value = make_start_token(name, MAPPING)
    context.write(tag('input', type='hidden', name='__start__', value=value, _closed=closed))
    context['caller'].body(state.mapping_descend(name))
    context.write(tag('input', type='hidden', name='__end__', value=value, _closed=closed))
    return ''


@supports_caller
def sequence_repeater(context, state, name=None, with_adder=False, with_removers=False):
    """
    Output html in a format for repetitions to be added to the sequence on the
    client side.

    The caller's body should receive a single argument: state.
    """
    closed = state.is_closed()
    repetition_css_class = _to_css_class(state.determine_css_classes('repetition'))
    #
    # Construct the repeater template and shove it into a div as a data
    # attribute.
    #
    template_state = state.clone(defaults=None, errors=None,
        id_stack=state.push_id_stack(name, '__REPEATER__'),
        getter=sequence_getter)
    template_str = [str(tag('div', class_=repetition_css_class, _closed=False)),
            capture(context, context['caller'].body, template_state)]
    if with_removers:
        remover_css_class = _to_css_class(state.determine_css_classes('remover'))
        template_str.append(str(tag('input', type_='button', class_=remover_css_class, _closed=closed, value='Remove')))
    template_str.append('</div>')
    template_str = ''.join(template_str)
    repeater_css_class = _to_css_class(state.determine_css_classes('repeater'))
    context.write(tag('div', class_=repeater_css_class, _closed=False, **{'data-template': template_str}))
    #
    # Optionally include a header.
    #
    if hasattr(context['caller'], 'header'):
        context['caller'].header()
    #
    # Loop, rendering each repetition and optionally a remove button.
    #
    value = make_start_token(name, SEQUENCE)
    context.write(tag('input', type='hidden', name='__start__', value=value,
            _closed=closed))
    repetitions_css_class = _to_css_class(state.determine_css_classes('repetitions'))
    # Wrap all the repetitions in another div.
    context.write(tag('div', class_=repetitions_css_class, _closed=False))
    for new_state in state.sequence_descend(name):
        # Wrap each repetition in a div.
        context.write(tag('div', class_=repetition_css_class, _closed=False))
        context['caller'].body(new_state)
        if with_removers:
            context.write(tag('input', type_='button', class_=remover_css_class, _closed=closed, value='Remove'))
        context.write('</div>')
    context.write('</div>')
    context.write(tag('input', type='hidden', name='__end__', value=value, _closed=closed))
    #
    # Optionally include an add button.
    #
    if with_adder:
        adder_css_class = _to_css_class(state.determine_css_classes('adder'))
        context.write(tag('input', type_='button', class_=adder_css_class, _closed=closed, value='Add'))
    #
    # Optionally include a footer.
    #
    if hasattr(context['caller'], 'footer'):
        context['caller'].footer()
    context.write('</div>')
    return ''


@supports_caller
def sequence(context, state, name=None):
    """
    Wrap a block of fields that should be serialized as a list using
        peppercorn.

    The caller's body should receive a single argument: state.

    This function kicks abstraction right in the face.
    """
    closed = state.is_closed()
    value = make_start_token(name, SEQUENCE)
    context.write(tag('input', type='hidden', name='__start__', value=value,
            _closed=closed))
    for new_state in state.sequence_descend(name):
        context['caller'].body(new_state)
    context.write(tag('input', type='hidden', name='__end__', value=value,
            _closed=closed))
    return ''



try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


install_requires = ["webhelpers", "mako", "peppercorn"]


setup(
    name='pepperedform',
    version='0.6.1',
    description='Helpers for using peppercorn with formprocess.',
    author='Ian Wilson',
    author_email='ian@laspilitas.com',
    license="BSD",
    url='',
    install_requires=install_requires,
    test_requires=install_requires + ['webob'],
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ]
)



"""
Test webhelper html wrappers.
"""
import unittest

from pepperedform.render import (text_field, checkbox_field,
        textarea_field, multiple_select_field, single_select_field, mapping,
        sequence, rename, primary_form_action, secondary_form_action,
        sequence_repeater, file_field)
from pepperedform.renderutils import FormRendererState, FormRenderer


class MockCallerStack(object):

    def _push_frame(*args, **kwargs):
        pass

    def _pop_frame(*args, **kwargs):
        pass


class MockContext(object):

    def _push_buffer(self, *args, **kwargs):
        # @TODO: Is this working ?
        self.buffers.append(self)

    def _pop_buffer(self):
        # @TODO: Is this working ?
        return self.buffers.pop()

    def __init__(self, caller=None):
        self.buffers = []
        self.attrs = {}
        self.out = []
        if caller:
            self.attrs['caller'] = caller
        self.caller_stack = MockCallerStack()

    def __getitem__(self, k):
        return self.attrs[k]

    def write(self, s):
        self.out.append(s)

    def getvalue(self):
        # @TODO: Is this working ? With push/pop buffer?
        return self.serialize()

    def serialize(self):
        return (u''.join(self.out)).encode('utf8')


class MockCaller(object):

    def __init__(self, body_text=u'', callback=None):
        self.body_text = body_text
        self.callback = callback

    def body(self, *args, **kwargs):
        if self.callback:
            self.callback(*args, **kwargs)
        else:
            return self.body_text

    def caller_stack(*args, **kwargs):
        pass


class TestRender(unittest.TestCase):

    def _get_mock_getter(self):
        def _mock_getter(items, name, if_missing=None):
            if items:
                return items.get(name, if_missing)
            return if_missing
        return _mock_getter

    def _get_form_renderer_state(self, defaults, errors=None):
        form = FormRenderer()
        return form.form_renderer_state(defaults, errors)

    def test_text_field_brainless(self):
        """
        Test text_field runs.
        """
        name = 'day'
        defaults = {
            name: '15'
        }
        context = MockContext()
        text_field(context, self._get_form_renderer_state(defaults), name)
        assert defaults[name] in context.serialize()

    def test_file_field_brainless(self):
        """
        Test file_field runs.
        """
        name = 'image'
        defaults = {}
        context = MockContext()
        file_field(context, self._get_form_renderer_state(defaults), name)
        assert name in context.serialize()

    def test_checkbox_field_brainless(self):
        name = 'is_today'
        defaults = {
            name: True
        }
        context = MockContext()
        checkbox_field(context, self._get_form_renderer_state(defaults), name)
        assert 'checked' in context.serialize()

    def test_textarea_field_brainless(self):
        name = 'coffee'
        defaults = {
            name: 'I think I will.'
        }
        context = MockContext()
        state = self._get_form_renderer_state(defaults)
        textarea_field(context, state, name)
        assert defaults[name] in context.serialize()

    def test_single_select_field_brainless(self):
        name = 'roast'
        selected_value = 'dark'
        defaults = {
            name: selected_value,
        }
        options = [(selected_value, 'Dark'), ('light', 'Light'), ('medium', 'Medium')]
        context = MockContext()
        state = self._get_form_renderer_state(defaults)
        single_select_field(context, state, name, options=options)
        html = context.serialize()
        assert 'selected=' in html, html

    def test_multiple_select_field_brainless(self):
        name = 'roast'
        defaults =  {
            name: ['dark', 'light']
        }
        options = [('dark', 'Dark'), ('light', 'Light'), ('medium', 'Medium')]
        context = MockContext()
        state = self._get_form_renderer_state(defaults)
        multiple_select_field(context, state, name, options=options)
        html = context.serialize()
        assert html.count('selected=') == 2, html

    def test_mapping_brainless(self):
        mapping_name = 'coffee'
        field_name = 'roast'
        field_value = 'dark'
        defaults = {
            mapping_name: {
                field_name: field_value
            }
        }
        # @TODO: Closure references context, this is pretty horrid.
        # We should clean up the mocks.
        def caller(state):
            text_field(context, state, field_name)
        context = MockContext(caller=MockCaller(callback=caller))
        state = self._get_form_renderer_state(defaults)
        mapping(context, state, mapping_name)
        html = context.serialize()
        assert mapping_name in html, html
        assert field_name in html, html
        assert field_value in html, html

    def test_sequence_brainless(self):
        """ Test that the sequence wrapper works. """
        sequence_name = 'coffees'
        defaults = {
            sequence_name: ['French Roast'],
        }
        values = {
            'counter': 0
        }
        def callback(state):
            values[values['counter']] = state.field_value(None)
            values['counter'] += 1
        context = MockContext(caller=MockCaller(callback=callback))
        state = self._get_form_renderer_state(defaults)
        sequence(context, state, sequence_name)
        html = context.serialize()
        assert sequence_name in html, html
        self.assertEquals(values[0], 'French Roast',
                'Only one value in list so value at 0 is French Roast')
        # callback should be called once.
        self.assertEquals(values['counter'], 1)

    def test_sequence_repeater_brainless(self):
        """ Test that the sequence repeater wrapper works. """
        sequence_name = 'coffees'
        defaults = {
            sequence_name: ['French Roast']
        }
        values = {
            'counter': 0
        }
        def callback(state):
            values[values['counter']] = state.field_value(None)
            values['counter'] += 1
        context = MockContext(caller=MockCaller(callback=callback))
        state = self._get_form_renderer_state(defaults)
        sequence_repeater(context, state, sequence_name)
        html = context.serialize()
        assert sequence_name in html, html
        self.assertEquals(values[0], None,
                'Template has no value so value at 0 is None.')
        self.assertEquals(values[1], 'French Roast',
                'Only one value in actual list so value at 1 is French Roast')
        # callback should be called twice.
        self.assertEquals(values['counter'], 2)

    def test_primary_form_action(self):
        """ Test that a primary form action is displayed. """
        state = self._get_form_renderer_state({})
        context = MockContext()
        value = u'Update'
        primary_form_action(context, state, value)
        assert value in context.serialize()

    def test_secondary_form_action(self):
        """ Test that a secondary form action is displayed. """
        context = MockContext()
        value = u'Cancel'
        state = self._get_form_renderer_state({})
        secondary_form_action(context, state, value)
        assert value in context.serialize()




"""
Test tools for using formprocess handlers.
"""
import unittest

from webob.multidict import MultiDict

from pepperedform.handler import parse_with_peppercorn, renderer_state_injector


class DummyHandler(object):
    pass


class DummyState(object):
    pass


class DummyFormRenderer(object):
    def form_renderer_state(self, *args, **kwargs):
        return dict(args=args, kwarg=kwargs)


class TestHandlerTools(unittest.TestCase):

    def test_parse_for_peppercorn(self):
        handler = DummyHandler()
        state = DummyState()
        unsafe_params = MultiDict([
            ('__start__', 'days:sequence'),
            ('__start__', 'day:sequence'),
            ('day', '1'),
            ('__end__', 'day:sequence'),
            ('__start__', 'sequence'),
            ('day', '2'),
            ('__end__', 'day:sequence'),
            ('__start__', 'day:sequence'),
            ('day', '3'),
            ('__end__', 'day:sequence'),
            ('__end__', 'days:sequence')])
        new_defaults = parse_with_peppercorn(handler, unsafe_params, state)
        self.assertEqual(len(new_defaults['days']), 3)

    def test_renderer_state_injector(self):
        form_renderer = DummyFormRenderer()
        injector = renderer_state_injector(form_renderer, inject_at_key='test')
        fill_kwargs = {}
        defaults = {}
        errors = {}
        fill_kwargs = injector(None, defaults, errors, None, fill_kwargs)
        self.assertEqual(fill_kwargs['test']['args'], (defaults, errors))



"""
Hooks for use with the Handler class in formprocess.
"""
from peppercorn import parse


def parse_with_peppercorn(handler_instance, unsafe_params, state):
    """
    Extract form params in the peppercorn format into structured params.

    Note: This should be used as a default_filter_hook in a formprocess
        handler.
    """
    return parse(unsafe_params.items())


def renderer_state_injector(form_renderer, inject_at_key='renderer_state'):
    """
    Wrap a closure around the given form_renderer that creates a form
    renderer state and places it in the fill kwargs at the given
    key.
    """
    def inject_renderer_state(handler_instance, defaults, errors, state,
        fill_kwargs):
        fill_kwargs[inject_at_key] = form_renderer.form_renderer_state(
            defaults, errors)
        return fill_kwargs
    return inject_renderer_state
